{%- macro escape_name(name) -%}
{% if name.starts_with('"') %}{{ name }}{% else %}"{{ name }}"{% endif %}
{%- endmacro -%}

.include "$lib/core.asm"
.include "$lib/std.asm"
.include "$lib/main.asm"

.define MEM_LOOP_I, CORE_MEM_MEM
.define MEM_LOOP_J, (MEM_LOOP_I + 1)

.define MEM_CALL_STACK_LEN, 16
.define MEM_CALL_STACK_PTR, (MEM_LOOP_J + 1)
.define MEM_CALL_STACK_END, (MEM_CALL_STACK_PTR + MEM_CALL_STACK_LEN)

.define MEM_ALLOC_PTR, MEM_CALL_STACK_END

.std_rset CORE_REG_A, MEM_CALL_STACK_PTR
.std_set MEM_CALL_STACK_PTR

.std_rset CORE_REG_A, (MEM_ALLOC_PTR + 1)
.std_set MEM_ALLOC_PTR

.jump_main

call_stack_jump:
  .std_get MEM_CALL_STACK_PTR
  tlr CORE_REG_A
  .std_rset CORE_REG_B, 1
  .std_alu CORE_ALU_ADD
  tlr CORE_REG_A
  tlr CORE_REG_B
  .std_set MEM_CALL_STACK_PTR

  tsr CORE_REG_D
  tlr CORE_REG_A
  tsr CORE_REG_B
  set
  tsr CORE_REG_C
  tlr CORE_REG_PC

return_call_stack_jump:
  .std_get MEM_CALL_STACK_PTR
  tlr CORE_REG_A
  tlr CORE_REG_C
  .std_rset CORE_REG_B, 1
  .std_alu CORE_ALU_SUB
  tlr CORE_REG_A
  .std_set MEM_CALL_STACK_PTR

  tsr CORE_REG_C
  get
  tlr CORE_REG_PC

.macro stack_transfer_alu
  .std_ld
  tlr CORE_REG_B
  .std_ld
  tlr CORE_REG_A
.endmacro

.macro call_stack_jump, call_stack_jump_arg_0_label, call_stack_jump_arg_1_offset
  .std_rset CORE_REG_C, call_stack_jump_arg_0_label
  .std_rset CORE_REG_D, (call_stack_jump_arg_1_offset + 7)
  ts call_stack_jump
  tlr CORE_REG_PC
.endmacro

.macro return_call_stack_jump
  .std_jump return_call_stack_jump
.endmacro

; data
  ; strings
  {%- for s in strings %}
    {% let bytes = s.data.as_bytes() -%}
    data_strings_{{ s.id }}:
      {% let name = snailquote::escape(s.data) -%}
      ; original  : {% call escape_name(name) %}
      ; size      : {{ bytes.len() }}
      {%- for byte in bytes %}
      .bytes {{ "0x{:02x}"|format(byte) }}
      {%- endfor %}
      data_strings_end_{{ s.id }}:
  {% endfor %}

; conditions
{%- for c in conditions %}
  ; condition: {{ c.id }}
    conditions_if_{{ c.id }}:
    {%- for ins in c.data.0 %}
      {{ ins.to_code() }}
    {%- endfor %}

    conditions_else_{{ c.id }}:
    {%- for ins in c.data.0 %}
      {{ ins.to_code() }}
    {%- endfor %}
{% endfor %}

; words
{%- for w in words %}
  words_{{ w.id }}:
    {% let name = snailquote::escape(w.data.0) -%}
    ; name : {% call escape_name(name) %}
    {%- for node in w.data.1 %}
    {{ node.to_code() }}
    {%- endfor %}
    .return_call_stack_jump
{% endfor %}

; main
.def_main
  ts NULL ; reset tmp
{% for node in main %}
  {{ node.to_code() }}
{%- endfor %}
.std_stop
